from pathlib import Path


def home_agnostic_path(path: str) -> str:
    """Replace the home directory in the given path with a tilde."""
    home = Path.home()
    path = Path(path)
    try:
        # Use relative_to to get the relative path to the home directory
        relative_path = path.relative_to(home)
        return f"~/{relative_path}"
    except ValueError:
        # The path is not a subpath of the home directory
        return str(path)



from pathlib import Path

from env_wrangler.utils import home_agnostic_path


def test_home_agnostic_path():
    home = Path.home()

    # Test with a path in the home directory
    path_in_home = home / "test"
    assert home_agnostic_path(str(path_in_home)) == "~/test"

    # Test with a path not in the home directory
    path_not_in_home = Path("/tmp/test")
    assert home_agnostic_path(str(path_not_in_home)) == str(path_not_in_home)



import json
from pathlib import Path

from dotenv import dotenv_values

from .constants import MASK_VALUE


def envs_to_dict(env_files: list[str]) -> dict:
    """Read env files, merge them and return a dict"""
    config = {}
    for env_file in env_files:
        config |= dotenv_values(env_file)
    return config


def save_dict_to_json_file(data: dict, file_path: Path) -> Path:
    """Save a dictionary to a JSON file (non-destructive)."""
    if not data:
        return None
    file_path = Path(file_path).expanduser()
    if file_path.exists():
        existing_data = json.loads(file_path.read_text())
        existing_data.update(data)
        data = existing_data
    # We ensure it is sorted before writing
    file_path.write_text(json.dumps(data, indent=2, sort_keys=True))
    return file_path


def save_dict_to_env_file(data: dict, file_path: str) -> Path:
    """Save a dictionary to an env file (non-destructive)."""
    if not data:
        return None
    file_path = Path(file_path).expanduser()
    if file_path.exists():
        existing_data = dict(
            line.split("=", 1) for line in file_path.read_text().splitlines() if line
        )
        existing_data.update(data)
        data = existing_data
    # We ensure it is sorted before writing
    env_content = "\n".join([f"{key}={value}" for key, value in sorted(data.items())])
    file_path.write_text(env_content)
    return file_path


def filter_keys_by_substring(input_dict: dict, words_to_keep: list[str]) -> dict:
    """
    Filters a dictionary to keep only keys that include any of the specified words.

    :param input_dict: Dictionary to be filtered.
    :param words_to_keep: List of words to check against the keys in the dictionary.
    """
    return {
        key: input_dict[key]
        for key in input_dict
        if any(word in key for word in words_to_keep)
    }


def remove_masked_values(input_dict: dict) -> dict:
    """
    Filters a dictionary to remove masked values.

    :param input_dict: Dictionary to be filtered.
    """
    return {key: value for key, value in input_dict.items() if value != MASK_VALUE}


def mask_sensitive_data_in_file(
    file_path: str, filter_keys: list, ignore_keys=None
) -> Path:
    """Mask sensitive data in a .env file.

    Args:
        file_path (str): The path to the .env file.
        filter_keys (list): The keys to mask in the .env file.
    """
    file_path = Path(file_path).expanduser()
    lines = file_path.read_text().splitlines()

    ignore_keys = ignore_keys or []

    masked_lines = []
    for line in lines:
        for check_key in filter_keys:
            key, _ = line.split("=", 1)
            if check_key in key and key not in ignore_keys:
                line = key + f"={MASK_VALUE}"  # noqa: PLW2901
        masked_lines.append(line)

    file_path.write_text("\n".join(masked_lines))
    return file_path


def unmask_sensitive_data_in_file(file_path: str, replacements: dict) -> Path:
    """Unmask sensitive data in a .env file.

    Args:
        file_path (str): The path to the file.
        replacements (dict): The dictionary of key-value pairs.
    """
    file_path = Path(file_path).expanduser()
    lines = file_path.read_text().splitlines()

    replaced_lines = []
    for line in lines:
        for key, value in replacements.items():
            if key in line.split("=", 1)[0]:
                line = line.split("=", 1)[0] + "=" + value  # noqa: PLW2901
        replaced_lines.append(line)

    file_path.write_text("\n".join(replaced_lines))
    return file_path


def json_to_env(json_file_path: str, env_file_path: str) -> Path:
    """Convert a JSON file to a .env file.

    Args:
        json_file_path (str): The path to the JSON file.
        env_file_path (str): The path to the .env file.
    """
    json_file_path = Path(json_file_path).expanduser()
    env_file_path = Path(env_file_path).expanduser()

    with json_file_path.open("r") as json_file:
        data = json.load(json_file)

    env_lines = [f"{key}={value}" for key, value in data.items()]
    env_file_path.write_text("\n".join(env_lines))
    return env_file_path



import configparser
from pathlib import Path

import pkg_resources
from env_wrangler.settings import copy_resource_file
from env_wrangler.settings import get_config_value_as_list


def test_copy_resource_file(tmp_path):
    # Create a temp data file in the package resources
    temp_data_file = pkg_resources.resource_filename("env_wrangler", "data/test.txt")

    source_file = Path(temp_data_file)
    source_file.parent.mkdir(parents=True, exist_ok=True)
    source_file.write_text("Test content")

    # Call copy_resource_file with the source file name and a destination path
    copy_resource_file("test.txt", str(tmp_path / "dest" / "test.txt"))

    # Check that the destination file exists and has the correct content
    dest_file = tmp_path / "dest" / "test.txt"
    assert dest_file.is_file()
    assert dest_file.read_text() == "Test content"

    source_file.unlink()


def test_get_config_value_as_list():
    # Create a ConfigParser object and add a section with a key
    config = configparser.ConfigParser()
    config.add_section("default")
    config.set("default", "key_words", "item1, item2, item3")

    result = get_config_value_as_list(config, "default", "key_words")
    assert result == ["item1", "item2", "item3"]



import json
import logging
from pathlib import Path

import click  # https://click.palletsprojects.com/

from .constants import DEFAULT_SECTION
from .constants import IGNORE_KEYS_SETTING
from .constants import KEY_WORDS_SETTING
from .core import envs_to_dict
from .core import filter_keys_by_substring
from .core import mask_sensitive_data_in_file
from .core import remove_masked_values
from .core import save_dict_to_env_file
from .core import save_dict_to_json_file
from .core import unmask_sensitive_data_in_file
from .settings import config
from .settings import get_config_value_as_list
from .utils import home_agnostic_path

logger = logging.getLogger(__name__)


def silent_echo(*args, **kwargs):
    pass


def file_error():
    click.secho(
        "Path is a file, not a directory. Please provide a directory.",
        fg="red",
        err=True,
    )


def common_options(func):
    """Decorator to add common options to a command."""
    func = click.option(
        "-p",
        "--path",
        required=True,
        type=click.Path(),
        help="Path to the .env file or a directory containing .env files.",
    )(func)
    return click.option("--verbose", is_flag=True, help="Enables verbose mode.")(func)


@click.command()
@common_options
@click.option(
    "--format",
    type=click.Choice(["both", "json", "env"], case_sensitive=False),
    help="The output format.",
)
def extract(path, verbose, format):  # noqa: A002
    """Extract secrets from the .env file(s) in the given directory into a separate file."""

    if not verbose:
        click.echo = silent_echo

    path = Path(path).expanduser()
    if path.is_file():
        file_error()
        return

    key_words = get_config_value_as_list(config, DEFAULT_SECTION, KEY_WORDS_SETTING)

    click.echo(f"Extracting secrets from all .env files in {home_agnostic_path(path)}")
    target_envs = get_config_value_as_list(config, DEFAULT_SECTION, "envs")
    env_files = [path / file for file in target_envs]
    env = envs_to_dict(env_files)

    secrets_dict = filter_keys_by_substring(env, key_words)
    secrets_dict = remove_masked_values(secrets_dict)

    if not secrets_dict:
        click.secho("No secrets found to extract.", err=True, fg="yellow")
        return

    if format == "json":
        output_file = save_dict_to_json_file(secrets_dict, path / "secrets.json")
        click.echo(f"Secrets saved to {home_agnostic_path(output_file)}")
    elif format == "env":
        output_file = save_dict_to_env_file(secrets_dict, path / ".secrets")
        click.echo(f"Secrets saved to {home_agnostic_path(output_file)}")
    else:
        output_file1 = save_dict_to_json_file(secrets_dict, path / "secrets.json")
        output_file2 = save_dict_to_env_file(secrets_dict, path / ".secrets")
        click.echo(f"Secrets saved to {home_agnostic_path(output_file1)}")
        click.echo(f"Secrets saved to {home_agnostic_path(output_file2)}")


@click.command()
@common_options
def mask(path, verbose) -> None:
    """Mask sensitive data in the .env file(s) in the given directory."""

    if not verbose:
        click.echo = silent_echo

    path = Path(path).expanduser()
    if path.is_file():
        file_error()
        return

    # Ensure that the secrets file exists
    secret_env = path / ".secrets"
    secret_json = path / "secrets.json"
    if not secret_env.exists() and not secret_json.exists():
        click.secho(
            (
                "No secrets file(s) found (.secrets or secrets.json). "
                "If you have not extracted secrets, please run 'extract' first.\n"
                "This prevents mistakenly masking all values in the .env file(s) without a backup."
            ),
            fg="yellow",
            err=True,
        )
        return

    key_words = get_config_value_as_list(config, DEFAULT_SECTION, KEY_WORDS_SETTING)
    ignore_keys = get_config_value_as_list(config, DEFAULT_SECTION, IGNORE_KEYS_SETTING)

    masked_files = []
    target_envs = get_config_value_as_list(config, DEFAULT_SECTION, "envs")
    for file_path in target_envs:
        file = path / file_path
        if file.exists():
            masked_files.append(file)
            mask_sensitive_data_in_file(file, key_words, ignore_keys)

    # Let the user know which files were masked
    if masked_files:
        click.echo("Masked sensitive data in the following envs:")
        for file in masked_files:
            click.echo(f"   {home_agnostic_path(file)}")


@click.command()
@common_options
def unmask(path, verbose) -> None:
    """Unmask sensitive data in the .env file(s) in the given directory."""

    if not verbose:
        click.echo = silent_echo

    path = Path(path).expanduser()
    if path.is_file():
        file_error()
        return

    # Ensure that the secrets file exists
    secret_env = path / ".secrets"
    secret_json = path / "secrets.json"
    if not secret_env.exists() and not secret_json.exists():
        click.secho(
            "No secrets file(s) found (.secrets or secrets.json).",
            fg="yellow",
            err=True,
        )
        return

    key_words = get_config_value_as_list(config, DEFAULT_SECTION, KEY_WORDS_SETTING)

    unmasked_files = []
    target_envs = get_config_value_as_list(config, DEFAULT_SECTION, "envs")
    for file_path in target_envs:
        env_file = path / file_path
        if env_file.exists():
            unmasked_files.append(env_file)
            if secret_env.exists():
                env = envs_to_dict([secret_env])
                filtered = filter_keys_by_substring(env, key_words)
            elif secret_json.exists():
                filtered = json.loads(secret_json.read_text())
            unmask_sensitive_data_in_file(env_file, filtered)

    # Let the user know which files were unmasked
    if unmasked_files:
        click.echo("Unmasked sensitive data in the following envs:")
        for file in unmasked_files:
            click.echo(f"   {home_agnostic_path(file)}")


# Set up your command-line interface grouping
@click.group()
@click.version_option()
def cli():
    """Extract secrets from .env files into their own file(s) for use in a
    3rd party secrets manager."""


cli.add_command(extract)
cli.add_command(mask)
cli.add_command(unmask)

if __name__ == "__main__":
    cli()



import logging
import shutil
from configparser import ConfigParser
from pathlib import Path

import pkg_resources

logger = logging.getLogger(__name__)

# -----------------------------------------------------------------------------


def copy_resource_file(filename, dst):
    """Copy data files from package data folder."""
    # Create destination dir if needed
    dir_path = Path(dst).parent
    if not dir_path.is_dir():
        dir_path.mkdir()

    # Copy data file to destination
    src = pkg_resources.resource_filename("env_wrangler", f"data/{filename}")
    dst = str(Path(dir_path).expanduser())
    shutil.copy2(src, dst)


CONFIG_FILE = Path("~/.env-wrangler/env-wrangler.cfg").expanduser()
if not CONFIG_FILE.exists():
    copy_resource_file("env-wrangler.cfg", str(CONFIG_FILE))  # pragma: no cover

LOG_FILE = Path("~/.env-wrangler/env-wrangler.log").expanduser()
if not LOG_FILE.exists():
    LOG_FILE.touch()  # pragma: no cover

# -----------------------------------------------------------------------------


def get_config_value_as_list(config: ConfigParser, section: str, key: str) -> list[str]:
    data_string = config[section][key]
    return [item.strip() for item in data_string.split(",")]


config = ConfigParser()
config.read(CONFIG_FILE)



import json
from pathlib import Path

from dotenv import dotenv_values
from env_wrangler.constants import MASK_VALUE
from env_wrangler.core import envs_to_dict
from env_wrangler.core import filter_keys_by_substring
from env_wrangler.core import json_to_env
from env_wrangler.core import mask_sensitive_data_in_file
from env_wrangler.core import remove_masked_values
from env_wrangler.core import save_dict_to_env_file
from env_wrangler.core import save_dict_to_json_file
from env_wrangler.core import unmask_sensitive_data_in_file


def test_envs_to_dict(tmp_path):
    # Create two .env files in the temporary directory
    (tmp_path / "file1.env").write_text("KEY1=VALUE1\nKEY2=VALUE2")
    (tmp_path / "file2.env").write_text("KEY3=VALUE3\nKEY4=VALUE4")

    # Call envs_to_dict with the paths to the .env files
    result = envs_to_dict([str(tmp_path / "file1.env"), str(tmp_path / "file2.env")])

    # Check that the result is a dictionary with the correct key-value pairs
    assert result == {
        "KEY1": "VALUE1",
        "KEY2": "VALUE2",
        "KEY3": "VALUE3",
        "KEY4": "VALUE4",
    }


def test_save_dict_to_json_file(tmp_path):
    data = {"key1": "value1", "key2": "value2"}
    file_path = tmp_path / "test.json"
    file_path.write_text(json.dumps(data, sort_keys=True))
    output_file = save_dict_to_json_file(data, str(file_path))

    file_path = Path(file_path)
    loaded_data = json.loads(file_path.read_text())

    assert isinstance(output_file, Path)
    assert loaded_data == data


def test_save_empty_dict_to_json_file(tmp_path):
    data = {}
    file_path = tmp_path / "test.json"
    output_file = save_dict_to_json_file(data, str(file_path))

    assert output_file is None


def test_save_dict_to_env_file(tmp_path):
    data = {
        "key1": "value1",
        "key2": "value2",
        "key3": "value3",
        "key4": "value4",
        "key5": "value==5",  # Test bug fix
    }
    file_path = tmp_path / "test.env"
    # Write only a few keys to the file to test that the function appends new keys
    file_path.write_text("key1=value1\nkey2=value2")
    output_file = save_dict_to_env_file(data, str(file_path))

    file_path = Path(file_path)
    lines = file_path.read_text().splitlines()

    loaded_data = {}
    for line in lines:
        key, value = line.split("=", 1)
        loaded_data[key] = value

    assert isinstance(output_file, Path)
    assert loaded_data == data


def test_save_empty_dict_to_env_file(tmp_path):
    data = {}
    file_path = tmp_path / "test.env"
    output_file = save_dict_to_env_file(data, str(file_path))

    assert output_file is None


def test_filter_keys_by_substring():
    input_dict = {
        "ACCESS_KEY": "my-access-key",
        "ACCESS_TOKEN": "my-access-token",
        "FOO": "bar",
        "BAR": "baz",
    }
    words_to_keep = ["ACCESS_KEY", "ACCESS_TOKEN"]

    result = filter_keys_by_substring(input_dict, words_to_keep)

    assert result == {
        "ACCESS_KEY": "my-access-key",
        "ACCESS_TOKEN": "my-access-token",
    }


def test_remove_masked_values():
    input_dict = {
        "key1": "value1",
        "key2": MASK_VALUE,
        "key3": "value3",
        "key4": MASK_VALUE,
    }
    expected_dict = {"key1": "value1", "key3": "value3"}

    assert remove_masked_values(input_dict) == expected_dict


def test_mask_sensitive_data_in_file(tmp_path):
    # Create a .env file in the temporary directory
    env_file = tmp_path / ".env"
    env_file.write_text(
        "SECRET_KEY=secret\nPASSWORD=password\nFOO=bar\nBAR=baz\nIGNORED_KEY=ignored"
    )

    # Call mask_sensitive_data with the path to the .env file and a list of sensitive keys
    output_file = mask_sensitive_data_in_file(
        str(env_file), ["KEY", "PASSWORD"], ["IGNORED_KEY"]
    )

    # Load the .env file and check that the sensitive keys have been masked
    env_vars = dotenv_values(str(env_file))
    assert isinstance(output_file, Path)
    assert env_vars["SECRET_KEY"] == MASK_VALUE  # noqa: S105
    assert env_vars["PASSWORD"] == MASK_VALUE  # noqa: S105
    assert env_vars["FOO"] == "bar"
    assert env_vars["BAR"] == "baz"
    assert env_vars["IGNORED_KEY"] == "ignored"


def test_unmask_sensitive_data_in_file(tmp_path):
    # Create a .env file in the temporary directory with masked sensitive data
    env_file = tmp_path / ".env"
    env_file.write_text(
        f"SECRET_KEY={MASK_VALUE}\nPASSWORD={MASK_VALUE}\nFOO=bar\nBAR=baz"
    )

    # Call unmask_sensitive_data_in_file with the path to the .env file, a list of sensitive keys, and their original values
    output_file = unmask_sensitive_data_in_file(
        str(env_file), {"SECRET_KEY": "secret", "PASSWORD": "password"}
    )

    # Load the .env file and check that the sensitive keys have been unmasked
    env_vars = dotenv_values(str(env_file))
    assert isinstance(output_file, Path)
    assert env_vars["SECRET_KEY"] == "secret"  # noqa: S105
    assert env_vars["PASSWORD"] == "password"  # noqa: S105
    assert env_vars["FOO"] == "bar"
    assert env_vars["BAR"] == "baz"


def test_json_to_env(tmp_path):
    # Create a JSON file in the temporary directory
    json_file = tmp_path / "data.json"
    data = {"SECRET_KEY": "secret", "PASSWORD": "password", "FOO": "bar", "BAR": "baz"}
    json_file.write_text(json.dumps(data))

    # Create a path for the .env file
    env_file = tmp_path / ".env"

    # Call json_to_env with the paths to the JSON and .env files
    output_file = json_to_env(str(json_file), str(env_file))

    # Load the .env file and check that it contains the correct data
    env_lines = env_file.read_text().splitlines()
    env_data = dict(line.split("=", 1) for line in env_lines)

    assert isinstance(output_file, Path)
    assert env_data == data
